@@ -1,5 +1,3 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
 #
 # Copyright 2002-2011 Zuza Software Foundation
 #
@@ -21,30 +19,33 @@
 """Classes for the support of Gettext .po and .pot files.
 
 This implementation assumes that cpo is working. This should not be used
-directly, but can be used once cpo has been established to work."""
+directly, but can be used once cpo has been established to work.
+"""
 
-#TODO:
+# TODO:
 # - handle headerless PO files better
 # - previous msgid and msgctxt
 # - accept only unicodes everywhere
 
-import re
 import copy
-import cStringIO
+import logging
+import re
 
-from translate.lang import data
 from translate.misc.multistring import multistring
-from translate.storage import pocommon, base, cpo, poparser
-from translate.storage.pocommon import encodingToUse
+from translate.storage import base, cpo, pocommon
+
+
+logger = logging.getLogger(__name__)
+
 
 lsep = " "
 """Separator for #: entries"""
 
-basic_header = r'''msgid ""
+basic_header = r"""msgid ""
 msgstr ""
 "Content-Type: text/plain; charset=UTF-8\n"
 "Content-Transfer-Encoding: 8bit\n"
-'''
+"""
 
 
 class pounit(pocommon.pounit):
@@ -55,22 +56,21 @@ class pounit(pocommon.pounit):
     # prev_msgid = []         #
     # prev_msgid_plural = []  #
     # typecomments = []       #   #, fuzzy
-    # msgidcomment = u""      #   _: within msgid
+    # msgidcomment = ""      #   _: within msgid
     # msgctxt
     # msgid = []
     # msgstr = []
 
     # Our homegrown way to indicate what must be copied in a shallow
     # fashion
-    __shallow__ = ['_store']
+    __shallow__ = ["_store"]
 
-    def __init__(self, source=None, encoding="UTF-8"):
-        pocommon.pounit.__init__(self, source)
-        self._encoding = encodingToUse(encoding)
+    def __init__(self, source=None, **kwargs):
+        super().__init__(source)
         self._initallcomments(blankall=True)
-        self._msgctxt = u""
+        self._msgctxt = ""
 
-        self.target = u""
+        self.target = ""
 
     def _initallcomments(self, blankall=False):
         """Initialises allcomments"""
@@ -79,58 +79,57 @@ class pounit(pocommon.pounit):
             self.automaticcomments = []
             self.sourcecomments = []
             self.typecomments = []
-            self.msgidcomment = u""
+            self.msgidcomment = ""
 
-    def getsource(self):
+    @property
+    def source(self):
         return self._source
 
-    def setsource(self, source):
+    @source.setter
+    def source(self, source):
         self._rich_source = None
-#        assert isinstance(source, unicode)
-        source = data.forceunicode(source or u"")
-        source = source or u""
+        source = source or ""
         if isinstance(source, multistring):
             self._source = source
-        elif isinstance(source, unicode):
+        elif isinstance(source, str):
             self._source = source
-        else:
-            #unicode, list, dict
+        else:  # If it is unicode, list or dict.
             self._source = multistring(source)
-    source = property(getsource, setsource)
 
-    def gettarget(self):
+    @property
+    def target(self):
         """Returns the unescaped msgstr"""
         return self._target
 
-    def settarget(self, target):
+    @target.setter
+    def target(self, target):
         """Sets the msgstr to the given (unescaped) value"""
         self._rich_target = None
-#        assert isinstance(target, unicode)
-#        target = data.forceunicode(target)
         if self.hasplural():
             if isinstance(target, multistring):
                 self._target = target
-            else:
-                #unicode, list, dict
+            else:  # If it is unicode, list or dict.
                 self._target = multistring(target)
         elif isinstance(target, (dict, list)):
             if len(target) == 1:
                 self._target = target[0]
             else:
-                raise ValueError("po msgid element has no plural but msgstr has %d elements (%s)" % (len(target), target))
+                raise ValueError(
+                    "po msgid element has no plural but msgstr"
+                    "has %d elements (%s)" % (len(target), target)
+                )
         else:
             self._target = target
-    target = property(gettarget, settarget)
 
     def getnotes(self, origin=None):
         """Return comments based on origin value (programmer, developer, source code and translator)"""
-        if origin == None:
-            comments = u"\n".join(self.othercomments)
-            comments += u"\n".join(self.automaticcomments)
+        if origin is None:
+            comments = "\n".join(self.othercomments)
+            comments += "\n".join(self.automaticcomments)
         elif origin == "translator":
-            comments = u"\n".join(self.othercomments)
+            comments = "\n".join(self.othercomments)
         elif origin in ["programmer", "developer", "source code"]:
-            comments = u"\n".join(self.automaticcomments)
+            comments = "\n".join(self.automaticcomments)
         else:
             raise ValueError("Comment type not valid")
         return comments
@@ -140,15 +139,14 @@ class pounit(pocommon.pounit):
         # ignore empty strings and strings without non-space characters
         if not (text and text.strip()):
             return
-        text = data.forceunicode(text)
         commentlist = self.othercomments
         autocomments = False
         if origin in ["programmer", "developer", "source code"]:
             autocomments = True
             commentlist = self.automaticcomments
-        if text.endswith(u'\n'):
+        if text.endswith("\n"):
             text = text[:-1]
-        newcomments = text.split(u"\n")
+        newcomments = text.split("\n")
         if position == "append":
             newcomments = commentlist + newcomments
         elif position == "prepend":
@@ -159,7 +157,7 @@ class pounit(pocommon.pounit):
         else:
             self.othercomments = newcomments
 
-    def removenotes(self):
+    def removenotes(self, origin=None):
         """Remove all the translator's notes (other comments)"""
         self.othercomments = []
 
@@ -170,7 +168,7 @@ class pounit(pocommon.pounit):
         # self.__shallow__
         shallow = set(self.__shallow__)
         # Make deep copies of all members which are not in shallow
-        for key, value in self.__dict__.iteritems():
+        for key, value in self.__dict__.items():
             if key not in shallow:
                 setattr(new_unit, key, copy.deepcopy(value))
         # Make shallow copies of all members which are in shallow
@@ -187,13 +185,13 @@ class pounit(pocommon.pounit):
 
     def _msgidlen(self):
         if self.hasplural():
-            len("".join([string for string in self.source.strings]))
+            len("".join(string for string in self.source.strings))
         else:
             return len(self.source)
 
     def _msgstrlen(self):
         if self.hasplural():
-            len("".join([string for string in self.target.strings]))
+            len("".join(string for string in self.target.strings))
         else:
             return len(self.target)
 
@@ -205,16 +203,7 @@ class pounit(pocommon.pounit):
         """
 
         def mergelists(list1, list2, split=False):
-            #decode where necessary
-            if unicode in [type(item) for item in list2] + [type(item) for item in list1]:
-                for position, item in enumerate(list1):
-                    if isinstance(item, str):
-                        list1[position] = item.decode("utf-8")
-                for position, item in enumerate(list2):
-                    if isinstance(item, str):
-                        list2[position] = item.decode("utf-8")
-
-            #Determine the newline style of list2
+            # Determine the newline style of list2
             lineend = ""
             if list2 and list2[0]:
                 for candidate in ["\n", "\r", "\n\r"]:
@@ -223,7 +212,7 @@ class pounit(pocommon.pounit):
                 if not lineend:
                     lineend = ""
 
-            #Split if directed to do so:
+            # Split if directed to do so:
             if split:
                 splitlist1 = []
                 splitlist2 = []
@@ -231,9 +220,9 @@ class pounit(pocommon.pounit):
                     splitlist1.extend(item.split())
                 for item in list2:
                     splitlist2.extend(item.split())
-                list1.extend([item for item in splitlist2 if not item in splitlist1])
+                list1.extend([item for item in splitlist2 if item not in splitlist1])
             else:
-                #Normal merge, but conform to list1 newline style
+                # Normal merge, but conform to list1 newline style
                 if list1 != list2:
                     for item in list2:
                         item = item.rstrip(lineend)
@@ -242,7 +231,7 @@ class pounit(pocommon.pounit):
                             list1.append(item)
 
         if not isinstance(otherpo, pounit):
-            super(pounit, self).merge(otherpo, overwrite, comments)
+            super().merge(otherpo, overwrite, comments)
             return
         if comments:
             mergelists(self.othercomments, otherpo.othercomments)
@@ -251,14 +240,19 @@ class pounit(pocommon.pounit):
                 # We don't bring across otherpo.automaticcomments as we consider ourself
                 # to be the the authority.  Same applies to otherpo.msgidcomments
                 mergelists(self.automaticcomments, otherpo.automaticcomments)
-#                mergelists(self.msgidcomments, otherpo.msgidcomments) #XXX?
+                #                mergelists(self.msgidcomments, otherpo.msgidcomments) #XXX?
                 mergelists(self.sourcecomments, otherpo.sourcecomments, split=True)
         if not self.istranslated() or overwrite:
             # Remove kde-style comments from the translation (if any). XXX - remove
             if pocommon.extract_msgid_comment(otherpo.target):
-                otherpo.target = otherpo.target.replace('_: ' + otherpo._extract_msgidcomments() + '\n', '')
+                otherpo.target = otherpo.target.replace(
+                    "_: " + otherpo._extract_msgidcomments() + "\n", ""
+                )
             self.target = otherpo.target
-            if self.source != otherpo.source or self.getcontext() != otherpo.getcontext():
+            if (
+                self.source != otherpo.source
+                or self.getcontext() != otherpo.getcontext()
+            ):
                 self.markfuzzy()
             else:
                 self.markfuzzy(otherpo.isfuzzy())
@@ -270,24 +264,35 @@ class pounit(pocommon.pounit):
                 self.markfuzzy()
 
     def isheader(self):
-        #TODO: fix up nicely
+        # TODO: fix up nicely
         return not self.getid() and len(self.target) > 0
 
     def isblank(self):
         if self.isheader() or self.msgidcomment:
             return False
-        if (self._msgidlen() == 0) and (self._msgstrlen() == 0) and len(self._msgctxt) == 0:
+        if (
+            (self._msgidlen() == 0)
+            and (self._msgstrlen() == 0)
+            and len(self._msgctxt) == 0
+        ):
             return True
         return False
 
     def hastypecomment(self, typecomment):
         """Check whether the given type comment is present"""
         # check for word boundaries properly by using a regular expression...
-        return sum(map(lambda tcline: len(re.findall("\\b%s\\b" % typecomment, tcline)), self.typecomments)) != 0
+        return (
+            sum(
+                map(
+                    lambda tcline: len(re.findall("\\b%s\\b" % typecomment, tcline)),
+                    self.typecomments,
+                )
+            )
+            != 0
+        )
 
     def hasmarkedcomment(self, commentmarker):
         """Check whether the given comment marker is present as # (commentmarker) ..."""
-#        raise DeprecationWarning
         commentmarker = "(%s)" % commentmarker
         for comment in self.othercomments:
             if comment.startswith(commentmarker):
@@ -301,11 +306,16 @@ class pounit(pocommon.pounit):
                 self.typecomments.append("#, %s\n" % typecomment)
             else:
                 # this should handle word boundaries properly ...
-                typecomments = map(lambda tcline: re.sub("\\b%s\\b[ \t,]*" % typecomment, "", tcline), self.typecomments)
-                self.typecomments = filter(lambda tcline: tcline.strip() != "#,", typecomments)
+                typecomments = map(
+                    lambda tcline: re.sub("\\b%s\\b[ \t,]*" % typecomment, "", tcline),
+                    self.typecomments,
+                )
+                self.typecomments = filter(
+                    lambda tcline: tcline.strip() != "#,", typecomments
+                )
 
     def istranslated(self):
-        return super(pounit, self).istranslated() and not self.isobsolete()
+        return super().istranslated() and not self.isobsolete()
 
     def istranslatable(self):
         return not (self.isheader() or self.isblank() or self.isobsolete())
@@ -320,17 +330,13 @@ class pounit(pocommon.pounit):
         """Makes this unit obsolete"""
         self.sourcecomments = []
         self.automaticcomments = []
-        super(pounit, self).makeobsolete()
+        super().makeobsolete()
 
     def hasplural(self):
         """returns whether this pounit contains plural strings..."""
         source = self.source
         return isinstance(source, multistring) and len(source.strings) > 1
 
-    def parse(self, src):
-        raise DeprecationWarning("Should not be parsing with a unit")
-        return poparser.parse_unit(poparser.ParseState(cStringIO.StringIO(src), pounit), self)
-
     def __str__(self):
         """convert to a string. double check that unicode is handled somehow here"""
         _cpo_unit = cpo.pounit.buildfromunit(self)
@@ -343,7 +349,7 @@ class pounit(pocommon.pounit):
         return: A list of the locations with '#: ' stripped
 
         """
-        #TODO: rename to .locations
+        # TODO: rename to .locations
         return self.sourcecomments
 
     def addlocation(self, location):
@@ -370,8 +376,7 @@ class pounit(pocommon.pounit):
         return self._msgctxt + self.msgidcomment
 
     def setcontext(self, context):
-        context = data.forceunicode(context or u"")
-        self._msgctxt = context
+        self._msgctxt = context or ""
 
     def getid(self):
         """Returns a unique identifier for this unit."""
@@ -380,23 +385,25 @@ class pounit(pocommon.pounit):
         # the msgid. For generation of .mo files, we might want to use this
         # code to generate the entry for the hash table, but for now, it is
         # commented out for conformance to gettext.
-#        id = '\0'.join(self.source.strings)
+        #        id = '\0'.join(self.source.strings)
         id = self.source
         if self.msgidcomment:
-            id = u"_: %s\n%s" % (context, id)
+            id = f"_: {context}\n{id}"
         elif context:
-            id = u"%s\04%s" % (context, id)
+            id = f"{context}\04{id}"
         return id
 
+    @classmethod
     def buildfromunit(cls, unit):
         """Build a native unit from a foreign unit, preserving as much
-        information as possible."""
+        information as possible.
+        """
         if type(unit) == cls and hasattr(unit, "copy") and callable(unit.copy):
             return unit.copy()
         elif isinstance(unit, pocommon.pounit):
             newunit = cls(unit.source)
             newunit.target = unit.target
-            #context
+            # context
             newunit.msgidcomment = unit._extract_msgidcomments()
             if not newunit.msgidcomment:
                 newunit.setcontext(unit.getcontext())
@@ -413,97 +420,58 @@ class pounit(pocommon.pounit):
             newunit.markfuzzy(unit.isfuzzy())
             if unit.isobsolete():
                 newunit.makeobsolete()
-            for tc in ['python-format', 'c-format', 'php-format']:
+            for tc in ["python-format", "c-format", "php-format"]:
                 if unit.hastypecomment(tc):
                     newunit.settypecomment(tc)
                     break
             return newunit
         else:
             return base.TranslationUnit.buildfromunit(unit)
-    buildfromunit = classmethod(buildfromunit)
 
 
 class pofile(pocommon.pofile):
     """A .po file containing various units"""
+
     UnitClass = pounit
 
-    def changeencoding(self, newencoding):
-        """Deprecated: changes the encoding on the file."""
-        # This should not be here but in poheader. It also shouldn't mangle the
-        # header itself, but use poheader methods. All users are removed, so
-        # we can deprecate after one release.
-        raise DeprecationWarning
-
-        self._encoding = encodingToUse(newencoding)
-        if not self.units:
-            return
-        header = self.header()
-        if not header or header.isblank():
-            return
-        charsetline = None
-        headerstr = header.target
-        for line in headerstr.split("\n"):
-            if not ":" in line:
-                continue
-            key, value = line.strip().split(":", 1)
-            if key.strip() != "Content-Type":
-                continue
-            charsetline = line
-        if charsetline is None:
-            headerstr += "Content-Type: text/plain; charset=%s" % self._encoding
-        else:
-            charset = re.search("charset=([^ ]*)", charsetline)
-            if charset is None:
-                newcharsetline = charsetline
-                if not newcharsetline.strip().endswith(";"):
-                    newcharsetline += ";"
-                newcharsetline += " charset=%s" % self._encoding
-            else:
-                charset = charset.group(1)
-                newcharsetline = charsetline.replace("charset=%s" % charset, "charset=%s" % self._encoding, 1)
-            headerstr = headerstr.replace(charsetline, newcharsetline, 1)
-        header.target = headerstr
-
     def _build_self_from_cpo(self):
         """Builds up this store from the internal cpo store.
 
         A user must ensure that self._cpo_store already exists, and that it is
-        deleted afterwards."""
+        deleted afterwards.
+        """
         for unit in self._cpo_store.units:
             self.addunit(self.UnitClass.buildfromunit(unit))
-        self._encoding = self._cpo_store._encoding
+        self.encoding = self._cpo_store.encoding
 
     def _build_cpo_from_self(self):
         """Builds the internal cpo store from the data in self.
 
         A user must ensure that self._cpo_store does not exist, and should
-        delete it after using it."""
+        delete it after using it.
+        """
         self._cpo_store = cpo.pofile(noheader=True)
         for unit in self.units:
             if not unit.isblank():
-                self._cpo_store.addunit(cpo.pofile.UnitClass.buildfromunit(unit, self._encoding))
+                self._cpo_store.addunit(
+                    cpo.pofile.UnitClass.buildfromunit(unit, self.encoding)
+                )
         if not self._cpo_store.header():
-            #only add a temporary header
-            self._cpo_store.makeheader(charset=self._encoding, encoding="8bit")
+            # only add a temporary header
+            self._cpo_store.makeheader(charset=self.encoding, encoding="8bit")
 
     def parse(self, input):
         """Parses the given file or file source string."""
         try:
-            if hasattr(input, 'name'):
+            if hasattr(input, "name"):
                 self.filename = input.name
-            elif not getattr(self, 'filename', ''):
-                self.filename = ''
-            tmp_header_added = False
-#            if isinstance(input, str) and '"Content-Type: text/plain; charset=' not in input[:200]:
-#                input = basic_header + input
-#                tmp_header_added = True
+            elif not getattr(self, "filename", ""):
+                self.filename = ""
             self.units = []
             self._cpo_store = cpo.pofile(input, noheader=True)
             self._build_self_from_cpo()
             del self._cpo_store
-            if tmp_header_added:
-                self.units = self.units[1:]
-        except Exception, e:
+        except Exception as e:
             raise base.ParseError(e)
 
     def removeduplicates(self, duplicatestyle="merge"):
@@ -519,6 +487,7 @@ class pofile(pocommon.pofile):
         def addcomment(thepo):
             thepo.msgidcomment = " ".join(thepo.getlocations())
             markedpos.append(thepo)
+
         for thepo in self.units:
             id = thepo.getid()
             if thepo.isheader() and not thepo.getlocations():
@@ -538,26 +507,32 @@ class pofile(pocommon.pofile):
                         origpo._msgctxt += " ".join(origpo.getlocations())
                         markedpos.append(thepo)
                     thepo._msgctxt += " ".join(thepo.getlocations())
-                    uniqueunits.append(thepo)
+                    if not thepo._msgctxt == id_dict[id]._msgctxt:
+                        uniqueunits.append(thepo)
+                    else:
+                        logger.warning(
+                            "Duplicate unit found with msgctx of '%s' and source '%s'",
+                            thepo._msgctxt,
+                            thepo.source,
+                        )
             else:
                 if not id:
                     if duplicatestyle == "merge":
                         addcomment(thepo)
                     else:
-                        thepo._msgctxt += u" ".join(thepo.getlocations())
+                        thepo._msgctxt += " ".join(thepo.getlocations())
                 id_dict[id] = thepo
                 uniqueunits.append(thepo)
         self.units = uniqueunits
 
-    def __str__(self):
-        """Convert to a string. double check that unicode is handled somehow here"""
-        self._cpo_store = cpo.pofile(encoding=self._encoding, noheader=True)
+    def serialize(self, out):
+        """Write content to file"""
+        self._cpo_store = cpo.pofile(encoding=self.encoding, noheader=True)
         try:
             self._build_cpo_from_self()
-        except UnicodeEncodeError, e:
-            self._encoding = "utf-8"
+        except UnicodeEncodeError:
+            self.encoding = "utf-8"
             self.updateheader(add=True, Content_Type="text/plain; charset=UTF-8")
             self._build_cpo_from_self()
-        output = str(self._cpo_store)
+        self._cpo_store.serialize(out)
         del self._cpo_store
-        return output
